import widgets
from constantes import *

_MESSAGES = (
    'HI HI, CETTE BASE\nPLANÉTAIRE EST À NOUS !',
    'BRAVO, VOUS AVEZ SURVÉCU À\nTOUTES LES VAGUES D\'ALIENS',
    'APPUYEZ SUR ENTRÉE\nPOUR REVENIR AU MENU'
)

class Menu(widgets.Interface) :
    ''' Interface du menu '''
    def __init__(self, parent, animation=True) :
        from menu import elements
        self._parent = parent
        super().__init__(parent)
        self._canevas = widgets.CanevasMenu(self)
        self._elements = elements.Elements(self._canevas, animation=animation)
        self._elements.afficher()
        delai = 200 if animation else 0
        self.after(self._elements.delai + delai, self._menu)

    def _menu(self) :
        '''
        Interne
        Affiche et active le menu
        '''
        from menu import menu as menu_
        commandes = (
            ('Jouer', self._parent.jeu),
            ('Scores', self._parent.score),
            ('Commandes', self._parent.commandes),
            ('Quitter', self._parent.destroy),
        )
        menu = menu_.Menu(self._canevas, commandes)
        menu.activer()


class Score(widgets.Interface) :
    ''' Interface du score '''
    def __init__(self, parent) :
        self._parent = parent
        from score import score as score_
        super().__init__(parent)
        self._canevas = widgets.CanevasScore(self)
        score = score_.Score(self._canevas)
        score.afficher()
        self._canevas.lien(
            'Appuyez sur entrée pour revenir au menu',
            HAUTEUR_ECRAN - 80,
            False,
        )
        self._canevas.bind('<Return>', lambda e: parent.menu())


class Commandes(widgets.Interface) :
    def __init__(self, parent) :
        from commandes import commandes
        self._parent = parent
        super().__init__(parent)
        self._canevas = widgets.CanevasCommandes(self)
        commandes.Commandes(self._canevas)
        self._canevas.lien(
            'Appuyez sur entrée pour revenir au menu',
            HAUTEUR_ECRAN - 80,
            False,
        )
        self._canevas.bind('<Return>', lambda e: parent.menu())


class Jeu(widgets.Interface) :
    ''' Interface du jeu '''
    def __init__(self, parent) :
        self._parent = parent
        from jeu import niveaux, messages, actions
        from jeu.infos import score, vies
        from jeu.elements import canon, blocs, aliens, elements, soucoupe
        super().__init__(parent)
        self._canevas = widgets.CanevasJeu(self)
        self._el = elements.Elements()
        self._el.clear()
        self._el.niveaux = niveaux.Niveaux(self._canevas)
        self._el.score = score.Score(self._canevas.infos)
        self._el.vies = vies.Vies(self._canevas.infos)
        self._el.canon = canon.Canon(self._canevas)
        self._el.blocs = blocs.Blocs(self._canevas.jeu)
        self._el.aliens = aliens.Aliens(self._canevas)
        self._el.jeu = self
        self._el.messages = messages.Messages(self._canevas.jeu)
        self._el.soucoupe = soucoupe.Soucoupe(self._canevas)
        self._el.blocs.creer()
        self._el.canon.creer()
        self._el.canon.projectile = False
        self._commandes = actions.Commandes(self._canevas.jeu)
        self._commandes.inscrire(
            CMD_TIRER,
            self._el.canon.tirer,
            spam=False
        )
        self._commandes.inscrire(
            CMD_GAUCHE,
            lambda : self._el.canon.deplacer(GAUCHE),
            exclusions=CMD_DROITE
        )
        self._commandes.inscrire(
            CMD_DROITE,
            lambda : self._el.canon.deplacer(DROITE),
            exclusions=CMD_GAUCHE
        )
        self._suspendu = False
        self._canevas.jeu.bind(
            '<KeyRelease-%s>' % CMD_PAUSE,
            lambda e: self._pause()
        )
        self._commandes.activer()
        self._el.niveaux.demarrer()

    def _pause(self) :
        self._suspendu = not self._suspendu
        if self._suspendu :
            self._attrsPause = (
                self._el.messages.texte,
                self._el.messages.actif,
                self._el.canon.projectile,
                self._el.canon.mouvement,
            )
            self._el.messages.texte = 'JEU EN PAUSE'
            self._el.messages.afficher()
            self._el.canon.projectile = False
            self._el.canon.mouvement = False
            self._canevas.jeu.suspendreActions()
        else :
            self._el.messages.texte = self._attrsPause[0]
            if not self._attrsPause[1] :
                self._el.messages.effacer()
            self._el.canon.projectile = self._attrsPause[2]
            self._el.canon.mouvement = self._attrsPause[3]
            self._canevas.jeu.reprendreActions()

    def terminer(self, gagne) :
        ''' Termine le jeu '''
        from score import fichier
        fichier.Fichier.enregistrer(self._el.score.score)
        self._el.soucoupe.apparition = False
        delai = int(
            (HAUTEUR_ZONE_JEU / PAS_PROJECTILE_CANON) *\
            DELAI_DEPLACEMENT_PROJECTILE_CANON
        )
        if not gagne :
            self._el.canon.detruire()
            self._el.aliens.projectile = False
            self._canevas.jeu.after(
                delai,
                setattr,
                self._el.aliens,
                'mouvement',
                False
            )
        else :
            self._canevas.jeu.after(
                delai,
                setattr,
                self._el.canon,
                'projectile',
                False
            )
        self._canevas.jeu.after(
            delai + 100, self._message, gagne)
        self._canevas.jeu.after(
            delai + 4000,
            self._afficherRetourMenu
        )

    def _afficherRetourMenu(self) :
        '''
        Interne
        Affiche et active le retour vers le menu principal
        '''
        self._commandes.radier()
        self._el.messages.texte = _MESSAGES[2]
        self._commandes.inscrire(
            'Return',
            self._parent.menu,
            spam=False
        )

    def _message(self, indice) :
        '''
        Interne
        Affiche le message relatif à l'indice fourni
        '''
        self._el.messages.texte = _MESSAGES[indice]
        self._el.messages.afficher()
