module MODInterfaces
    require './widgets'
    require './constantes'
    MESSAGES = [
        "HI HI, CETTE BASE\nPLANÉTAIRE EST À NOUS !",
        "BRAVO, VOUS AVEZ SURVÉCU À\nTOUTES LES VAGUES D'ALIENS",
        "APPUYEZ SUR ENTRÉE\nPOUR REVENIR AU MENU"
    ]

    class Menu < MODWidgets::Interface
        # Interface du menu
        def initialize parent, animation:true
            require './menu/menu'
            require './menu/elements'
            @parent = parent
            super parent
            @canevas = MODWidgets::CanevasMenu.new self
            @elements = MODMenu::Elements.new @canevas, animation:animation
            @elements.afficher
            delai = animation ? 200 : 0
            @canevas.after @elements.delai + delai, lambda {menu}
        end

        private
        def menu
            # Affiche et active le menu
            commandes = [
                ['jouer', lambda {@parent.jeu}],
                ['scores', lambda {@parent.score}],
                ['Commandes', lambda {@parent.commandes}],
                ['quitter', lambda {@parent.destroy}],
            ].freeze
            menu = MODMenu::Menu.new @canevas, commandes
            menu.activer
        end
    end


    class Score < MODWidgets::Interface
        def initialize parent
            require './score/score'
            @parent = parent
            super parent
            @canevas = MODWidgets::CanevasScore.new self
            score = MODScore::Score.new @canevas
            score.afficher
            @canevas.lien(
                'Appuyez sur entrée pour revenir au menu',
                HAUTEUR_ECRAN - 80,
            )
            @canevas.bind('Return', lambda {|e| @parent.menu})
        end
    end


    class Commandes < MODWidgets::Interface
        def initialize parent
            require './commandes/commandes'
            super parent
            canevas = MODWidgets::CanevasCommandes.new self
            MODCommandes::Commandes.new canevas
            canevas.lien(
                'Appuyez sur entrée pour revenir au menu',
                HAUTEUR_ECRAN - 80,
            )
            canevas.bind('Return', lambda {|e| parent.menu})
        end
    end


    class Jeu < MODWidgets::Interface
        def initialize parent
            ['actions', 'messages', 'niveaux'].each{|n|
                require "./jeu/#{n}"
            }
            ['aliens', 'blocs', 'canon', 'elements', 'soucoupe'].each{|n|
                require "./jeu/elements/#{n}"
            }
            require './jeu/infos/score'
            require './jeu/infos/vies'
            require './score/fichier'
            @parent = parent
            super parent
            @canevas = MODWidgets::CanevasJeu.new self
            @el = MODJeuElements::Elements.new
            @el.clear
            @el.niveaux = MODJeu::Niveaux.new @canevas
            @el.score = MODJeuInfos::Score.new @canevas.infos
            @el.vies = MODJeuInfos::Vies.new @canevas.infos
            @el.canon = MODJeuElements::Canon.new @canevas
            @el.blocs = MODJeuElements::Blocs.new @canevas.jeu
            @el.aliens = MODJeuElements::Aliens.new @canevas
            @el.messages = MODJeu::Messages.new @canevas.jeu
            @el.soucoupe = MODJeuElements::Soucoupe.new @canevas
            @el.jeu = self
            @el.blocs.creer
            @el.canon.creer
            @el.canon.projectile = false
            @commandes = MODJeu::Commandes.new @canevas.jeu
            @commandes.inscrire(
                CMD_TIRER,
                lambda {@el.canon.tirer},
                spam:false
            )
            @commandes.inscrire(
                CMD_GAUCHE,
                lambda {@el.canon.deplacer(GAUCHE)},
                exclusions:CMD_DROITE
            )
            @commandes.inscrire(
                CMD_DROITE,
                lambda {@el.canon.deplacer(DROITE)},
                exclusions:CMD_GAUCHE
            )
            @suspendu = false
            @canevas.jeu.bind(
                "KeyRelease-#{CMD_PAUSE}",
                lambda {|e| pause()}
            )
            @commandes.activer
            @el.niveaux.demarrer
        end

        def pause
            @suspendu = !@suspendu
            if @suspendu
                @attrsPause = [
                    @el.messages.texte,
                    @el.messages.actif?,
                    @el.canon.projectile,
                    @el.canon.mouvement,
                ]
                @el.messages.texte = 'JEU EN PAUSE'
                @el.messages.afficher
                @el.canon.projectile = false
                @el.canon.mouvement = false
                @canevas.jeu.suspendreActions
            else
                @el.messages.texte = @attrsPause[0]
                @el.messages.effacer unless @attrsPause[1]
                @el.canon.projectile = @attrsPause[2]
                @el.canon.mouvement = @attrsPause[3]
                @canevas.jeu.reprendreActions
            end
        end
        private :pause

        def terminer gagne
            MODScore::Fichier.enregistrer @el.score.score
            @el.soucoupe.apparition = false
            delai = (
                (HAUTEUR_ZONE_JEU / PAS_PROJECTILE_CANON) *
                DELAI_DEPLACEMENT_PROJECTILE_CANON
            )
            unless gagne
                @el.canon.detruire
                @el.aliens.projectile = false
                @canevas.jeu.after(
                    delai,
                    lambda {@el.aliens.mouvement = false}
                )
            else
                @canevas.jeu.after(
                    delai,
                    lambda {@el.canon.projectile = false}
                )
            end
            i = gagne ? 1 : 0
            @canevas.jeu.after(delai + 100, lambda {message(i)})
            @canevas.jeu.after(delai + 4000, lambda {afficherRetourMenu})
        end

        private
        def afficherRetourMenu
            @commandes.radier
            @el.messages.texte = MESSAGES[2]
            @commandes.inscrire(
                'Return',
                lambda {@parent.menu},
                spam:false
            )
        end

        def message indice
            @el.messages.texte = MESSAGES[indice]
            @el.messages.afficher
        end
    end
end
