class Commandes :
    ''' Gère les commandes clavier du jeu, pour :
        - Gestion des événements clavier simultanés
        - Fluidité des actions joueur
    '''
    def __init__(self, widget, delai=10) :
        '''
        @delai  : delai en ms entre les validations des événements
        '''
        self._widget = widget
        self._delai = delai
        self._touches = set()
        self._commandes = dict()

    def _appui(self, event) :
        ''' Interne '''
        if event.keysym in self._commandes :
            self._touches.add(event.keysym)

    def _relachement(self, event) :
        ''' Interne '''
        try :
            self._touches.remove(event.keysym)
        except KeyError :
            pass

    def inscrire(self, sym, commande, spam=True, exclusions=None) :
        '''
        Inscrit un événement clavier à gérer
        @sym        : symbole de la touche clavier
        @commande   : commande à exécuter à chaque validation
        @spam       : spécifie si la validation
        @exclusions : symboles, exécution de la commande si autres touches
                      non actives
        '''
        tex = type(exclusions)
        if tex == str or exclusions is None :
            exclusions = set((exclusions,))
        elif tex in (tuple, list, set) :
            exclusions = set(exclusions)
        else :
            TypeError('exclusions de types {} non gérés'.format(tex))
        self._commandes[sym] = (commande, spam, set(exclusions))

    def radier(self, sym=None) :
        '''
        Supprime la touche sym des validations et commande à exécuter
        @sym    : symbole de la touche clavier, si à None, supprimera toutes
                  celles inscrites
        '''
        if sym is not None :
            del(self._commandes[sym])
        else :
            self._commandes.clear()

    def _cvalider(self) :
        ''' Interne '''
        for sym, attrs in self._commandes.items() :
            if sym in self._touches and not self._touches & attrs[2] :
                attrs[0]()
            if not attrs[1] :
                try :
                    self._touches.remove(sym)
                except KeyError :
                    pass
        self._widget.after(self._delai, self._valider)

    def activer(self) :
        ''' Active la validation des événements clavier '''
        self._valider = self._cvalider
        if not hasattr(self, '_binds') :
            self._binds = (
                self._widget.bind('<KeyPress>', self._appui),
                self._widget.bind('<KeyRelease>', self._relachement),
            )
        self._valider()

    def desactiver(self) :
        ''' Désactive la validation des événements clavier '''
        self._valider = lambda : None
