module MODJeu
    class Commandes
        # Gère les commandes clavier du jeu, pour :
        #    - Gestion des événements clavier simultanés
        #    - Fluidité des actions joueur
        require 'set'
        def initialize widget, delai:10
            # @delai  : delai en ms entre les validations des événements
            @widget = widget
            @delai = delai
            @touches = Set.new
            @commandes = {}
            @procs = {
                :appui=>lambda {|e| appui(e)},
                :relachement=>lambda {|e| relachement(e)},
            }
        end

        def inscrire sym, commande, spam:true, exclusions:nil
            # Inscrit un événement clavier à gérer
            # @sym        : symbole de la touche clavier
            # @commande   : commande à exécuter à chaque validation
            # @spam       : spécifie si la validation
            # @exclusions : symboles, exécution de la commande si autres touches
            #               non actives
            tex = exclusions.class
            if tex == String or tex == NilClass
                exclusions = [exclusions]
                tex = Array
            end
            if tex == Array
                exclusions = Set.new(exclusions)
            end
            tex = exclusions.class
            if tex != Set
                msg = "exclusions de type #{tex} non géré"
                raise TypeError.new msg
            end
            @commandes[sym] = commande, spam, exclusions
        end

        def radier sym:nil
            # Supprime la touche sym des validations et commande à exécuter
            # @sym    : symbole de la touche clavier, si à None, supprimera toutes
            #           celles inscrits
            if sym != nil
                @commandes.delete sym
            else
                @commandes.clear
            end
        end

        def activer
            # Active la validation des événements clavier
            @valider = lambda {cvalider}
            if @binds == nil # Beurk
                @binds = [
                    @widget.bind('KeyPress', @procs[:appui]),
                    @widget.bind('KeyRelease', @procs[:relachement]),
                ]
            end
            @valider.call
        end

        def desactiver
            # Désactive la validation des événements clavier
            @valider = lambda {}
        end

        private
        def appui event
            if @commandes.has_key? event.keysym
                @touches.add event.keysym
            end
        end

        def relachement event
            @touches.delete event.keysym
        end

        def cvalider
            @commandes.each {|sym, attrs|
                if @touches.include? sym and (@touches & attrs[2]).empty?
                    attrs[0].call
                end
                unless attrs[1]
                     @touches.delete sym
                end
            }
            @widget.after @delai, @valider
        end
    end
end
