import random
import ressources
from jeu import collisions
from . import projectile
from constantes import *

class Alien :
    ''' Item alien '''
    def __init__(self, canevas, x, y, couleur, points) :
        self._images = ressources.Images()
        self._canevas = canevas
        self._couleur = couleur
        self._points = points
        self._image = self._images.image('alien_' + couleur)

        self._id = canevas.jeu.create_image(
            x, y,
            image=self._image,
            tags=TAG_ALIEN,
        )

    @property
    def id(self) :
        ''' Retourne l'identifiant alien sur le canevas '''
        return self._id

    @property
    def coord(self) :
        ''' Retourne la coordonnée centrale de l'alien '''
        return self._canevas.jeu.coords(self._id)

    @property
    def coords(self) :
        ''' Retourne les coordonnées de l'alien '''
        return self._canevas.jeu.bbox(self._id)

    @property
    def points(self) :
        ''' Retourne le nombre de points que vaut l'alien '''
        return self._points

    @property
    def couleur(self) :
        ''' Retourne la couleur de l'alien '''
        return self._couleur

    def detruire(self) :
        ''' Détruit l'alien en le faisant exploser '''
        img = self._images.image('alien_' + self._couleur, 2)
        self._canevas.jeu.itemconfigure(self._id, image=img)
        self._canevas.jeu.after(
            TEMPS_EXPLOSION_ALIEN,
            self._canevas.jeu.delete,
            self._id
        )

    def changerImage(self, indice) :
        ''' Change l'image de l'alien '''
        self._image = self._images.image('alien_' + self._couleur, indice)
        self._canevas.jeu.itemconfigure(
            self._id,
            image=self._image
        )


class Aliens :
    ''' Tableau des aliens '''
    def __init__(self, canevas) :
        self._canevas = canevas
        self._projectile = ProjectileAlien(canevas, self.tirer)
        self._aliens = []
        self._vivants = 0
        self._statuts = dict(
            projectile=True,
            mouvement=True,
        )
        self._vitesse = 500
        # ind., cmpt, max
        self._indicesImages = [0, 0, int(50 / PAS_ALIEN)]
        self.__direction = dict(x=PAS_ALIEN, y=0, vertical=0, etape=0)
        self._direction = self.__direction.copy()
        self._coli = 0

    @property
    def vivants(self) :
        ''' Retourne le nombre d'aliens vivants '''
        return self._vivants

    vitesse = property(
        lambda self: self._vitesse,
        lambda self, v: setattr(self, '_vitesse', int(v))
    )

    vitesseProjectile = property(
        lambda self: self._projectile.vitesse,
        lambda self, v: setattr(self._projectile, 'vitesse', v)
    )

    frequenceProjectile = property(
        lambda self: self._projectile.frequence,
        lambda self, v: setattr(self._projectile, 'frequence', v)
    )

    def _creerRangee(self, x, y, nombre, couleur) :
        ''' Interne '''
        points = POINTS_ALIENS[couleur]
        for col in range(nombre) :
            alien = Alien(self._canevas, x, y, couleur, points)
            self._aliens[col].append(alien)
            x += 50

    def creer(self, nbColonnes) :
        ''' Crée le tableau d'aliens '''
        self._aliens.clear()
        self._aliens.extend([] for _ in range(nbColonnes))
        self._direction.update(self.__direction)
        self._vivants = nbColonnes * len(COULEURS_ALIENS)
        y = ALIENS_Y
        for couleur in COULEURS_ALIENS[::-1] :
            self._creerRangee(20, y, nbColonnes, couleur)
            y += 45

    def detruire(self, alienId) :
        ''' Détruit un alien du tableau '''
        for col in self._aliens :
            for alien in col :
                if alien.id == alienId :
                    alien.detruire()
                    col.remove(alien)
                    self._vivants -= 1
                    return

    def points(self, alienId) :
        ''' Retourne le nombre de points de l'alien spécifié '''
        for col in self._aliens :
            for alien in col :
                if alien.id == alienId :
                    return alien.points
        print('alien {} supprimé...'.format(alienId))
        return 0

    def couleur(self, alienId) :
        ''' Retourne la couleur de l'alien spécifié '''
        for col in self._aliens :
            for alien in col :
                if alien.id == alienId :
                    return alien.couleur

    def tirer(self) :
        ''' Fait tirer les aliens '''
        if not self._statuts['projectile'] :
            return
        aliens = tuple(n for col in self._aliens for n in col)
        try :
            alien = random.choice(aliens)
        except IndexError :
            return
        self._projectile.creer(*alien.coord)

    def deplacer(self) :
        ''' Déplace le tableau d'aliens '''
        if not self._vivants or not self._statuts['mouvement'] :
            return
        # Test de collision et changement image chaque demie largeur
        if not self._coli % 20 :
            aliens = tuple(rangee[-1] for rangee in self._aliens if rangee)
            collisions.aliens(self._canevas, aliens)
            self._indicesImages[1] += 1
            if self._indicesImages[1] >= self._indicesImages[2] :
                self._indicesImages[0] += 1
                self._indicesImages[1] = 0
                self._changerImages()
            self._coli = -1
        self._coli += 1
        self._canevas.jeu.move(
            TAG_ALIEN,
            self._direction['x'],
            self._direction['y']
        )
        self._actualiserDeplacement()
        self._canevas.jeu.after(
            self._vitesse,
            self.deplacer,
        )

    def _actualiserDeplacement(self) :
        ''' Interne '''
        SEMI_VERTICAL = int(PAS_ALIEN_VERTICAL / 2)
        drt = self._direction
        if not drt['y'] :
            if drt['x'] > 0 :
                x = 0
                for col in self._aliens :
                    try :
                        alien = col[-1]
                    except IndexError :
                        continue
                    coord = alien.coord
                    if coord[0] > x :
                        x = coord[0]
                x += 20
                if x + PAS_ALIEN >= LARGEUR_ECRAN - SEMI_VERTICAL :
                    drt['y'] = PAS_ALIEN
                    drt['etape'] = 1
            else :
                x = LARGEUR_ECRAN
                for col in self._aliens :
                    try :
                        alien = col[0]
                    except IndexError :
                        continue
                    coord = alien.coord
                    if coord[0] < x :
                        x = coord[0]
                x -= 20
                if x - PAS_ALIEN <= SEMI_VERTICAL :
                    drt['y'] = PAS_ALIEN
                    drt['etape'] = 1
        else :
            if drt['etape'] == 1 :
                if drt['vertical'] < SEMI_VERTICAL :
                    drt['vertical'] += PAS_ALIEN
                else :
                    drt['etape'] = 2
                    drt['x'] = -PAS_ALIEN if drt['x'] > 0 else PAS_ALIEN
            else :
                if drt['vertical'] > 0 :
                    drt['vertical'] -= PAS_ALIEN
                else :
                    drt['y'] = 0
                    drt['vertical'] = 0

    def _changerImages(self) :
        ''' Interne '''
        indice = self._indicesImages[0] % 2
        for col in self._aliens :
            for alien in col :
                alien.changerImage(indice)

    def _peutBouger(self, booleen) :
        ''' Interne '''
        booleen = bool(booleen)
        if booleen == self._statuts['mouvement'] :
            return
        self._statuts['mouvement'] = booleen
        if booleen :
            self.deplacer()

    def _peutTirer(self, booleen)  :
        ''' Interne '''
        booleen = bool(booleen)
        if booleen == self._statuts['projectile'] :
            return
        self._statuts['projectile'] = booleen
        if booleen :
            self.tirer()

    mouvement = property(fset=_peutBouger)
    projectile = property(fset=_peutTirer)


class ProjectileAlien(projectile.Projectile) :
    ''' Projectile alien '''
    def __init__(self, canevas, tirer) :
        super().__init__(canevas, C_PROJECTILE_ALIEN, TAG_PROJECTILE_ALIEN)
        self._tirer = tirer
        self._vitesse = 50
        self._frequence = 500

    def deplacer(self) :
        ''' Déplace le projectile vers le bas '''
        coords = self.coords
        col = collisions.projectileAlien(self._canevas, *coords)
        if not col and coords[1] < HAUTEUR_ZONE_JEU :
            self._canevas.jeu.move(self._id, 0, 4)
            self._canevas.jeu.after(
                self._vitesse,
                self.deplacer
            )
        else:
            self.detruire()
            self._canevas.jeu.after(
                self._frequence,
                self._tirer,
            )

    vitesse = property(
        lambda self: self._vitesse,
        lambda self, v: setattr(self, '_vitesse', int(v))
    )
    frequence = property(
        lambda self: self._frequence,
        lambda self, v: setattr(self, '_frequence', int(v))
    )
