module MODJeuElements
    require './jeu/elements/projectile'
    require './jeu/collisions'
    require './constantes'
    class ProjectileAlien < MODJeuElements::Projectile
        # Projectile alien
        attr_reader :vitesse, :frequence
        def initialize canevas, tirer
            super canevas, C_PROJECTILE_ALIEN, TAG_PROJECTILE_ALIEN
            @tirer = tirer
            @vitesse = 30
            @frequence = 600
        end

        def deplacer
            # Déplace le projectile vers le bas
            c = coords
            col = MODJeu::Collisions.projectileAliens @canevas, *c
            unless col or c[1] > HAUTEUR_ZONE_JEU
                @canevas.jeu.move @id, 0, 4
                @canevas.jeu.after @vitesse, @deplacer
            else
                detruire
                @canevas.jeu.after @frequence, @tirer
            end
        end

        def vitesse=(int)
            @vitesse = int
        end

        def frequence=(int)
            @frequence = int
        end
    end


    class Alien
        require './ressources'
        attr_reader :id, :couleur, :points
        def initialize canevas, x, y, couleur, points
            @canevas = canevas
            @coord = x, y
            @couleur = couleur
            @points = points
            @images = MODRessources::Images.new
            @image = @images.image "alien_#{@couleur}"
            creer
        end

        def creer
            @id = @canevas.jeu.create(
                :image,
                *@coord,
                :image=>@image,
                :tags=>TAG_ALIEN,
            )
        end
        private :creer

        def coord
            # coordonnée centrale de l'image
            @canevas.jeu.coords @id
        end

        def coords
            @canevas.jeu.bbox @id
        end

        def detruire
            # Détruit l'alien en le faisant exploser
            self.image = 2
            @canevas.jeu.after(
                TEMPS_EXPLOSION_ALIEN,
                lambda {@canevas.jeu.delete(@id)}
            )
        end

        def image=(indice)
            img = @images.image "alien_#{@couleur}", indice:indice
            @canevas.jeu.itemconfigure(@id, :image=>img)
        end
    end


    class Aliens
        attr_reader :vivants, :vitesse
        # Tableau des aliens
        def initialize canevas
            @canevas = canevas
            @projectile = ProjectileAlien.new @canevas, lambda {tirer}
            @aliens = []
            @vivants = 0
            @statuts = {
                :projectile=>false,
                :mouvement=>false,
            }
            @vitesse = 0
            # ind., cmpt, max
            @indImages = [0, 0, 50 / PAS_ALIEN]
            @_direction = {:x=>PAS_ALIEN, :y=>0, :vertical=>0, :etape=>0}
            @direction = @_direction.clone
            @_direction.freeze
            @SEMI_VERTICAL = PAS_ALIEN_VERTICAL / 2
            @procs = {:deplacer=>lambda {deplacer},}
        end

        def vitesse=(int)
            @vitesse = int.to_i
        end

        def vitesseProjectile
            @projectile.vitesse
        end

        def vitesseProjectile=(int)
            @projectile.vitesse = int
        end

        def frequenceProjectile
            @projectile.frequence
        end

        def frequenceProjectile=(int)
            @projectile.frequence = int
        end

        def mouvement=(bool)
            return if bool == @statuts[:mouvement]
            @statuts[:mouvement] = bool
        end

        def projectile=(bool)
            return if bool == @statuts[:projectile]
            @statuts[:projectile] = bool
        end

        def points alienId
            # Retourne le nombre de points de l'alien spécifié
            @aliens.each {|col|
                col.each {|alien|
                    return alien.points if alien.id == alienId
                }
            }
            p "points indéterminés, alien #{alienId} supprimé..."
            return 0
        end

        def couleur alienId
            # Retourne la couleur de l'alien spécifié
            @aliens.each {|col|
                col.each {|alien|
                    return alien.couleur if alien.id == AlienId
                }
            }
        end

        def creer nbColonnes
            # Crée le tableau d'aliens
            @icol = 0
            @aliens.clear
            (0...nbColonnes).each {@aliens << []}
            @direction.update @_direction
            @vivants = nbColonnes * 5
            y = ALIENS_Y
            COULEURS_ALIENS.reverse.each {|couleur|
                creerRangee 20, y, nbColonnes, couleur
                y += 45
            }
        end

        def detruire alienId
            @aliens.each {|col|
                col.each {|alien|
                    if alien.id == alienId
                        alien.detruire
                        col.delete alien
                        @vivants -= 1
                        return
                    end
                }
            }
        end

        def tirer
            # Fait tirer les aliens
            return unless @statuts[:projectile]
            aliens = []
            @aliens.each {|col| col.each {|alien| aliens << alien}}
            return if aliens.empty? # Devrait pas se produire
            ind = rand aliens.length
            @projectile.creer *aliens[ind].coord
        end

        def deplacer
            _deplacer
        end

        private
        def _deplacer
            # Déplace le tableau d'aliens
            return unless @vivants !=0 and @statuts[:mouvement]
            if @icol % 20 == 0
                aliens = []
                @aliens.each {|col| aliens << col[-1] unless col.empty?}
                MODJeu::Collisions.aliens @canevas, aliens
                @indImages[1] += 1
                if @indImages[1] >= @indImages[2]
                    @indImages[0] += 1
                    @indImages[1] = 0
                    changerImages
                end
                @icol = -1
            end
            @icol += 1
            @canevas.jeu.move TAG_ALIEN, @direction[:x], @direction[:y]
            actualiserDeplacement
            @canevas.jeu.after @vitesse, @procs[:deplacer]
        end

        def actualiserDeplacement
            if @direction[:y] == 0
                if @direction[:x] > 0
                    x = 0
                    @aliens.each {|col|
                        next if col.empty?
                        coord = col[-1].coord
                        x = coord[0] if coord[0] > x
                    }
                    x += 20
                    if x + PAS_ALIEN >= LARGEUR_ECRAN - @SEMI_VERTICAL
                        @direction[:y] = PAS_ALIEN
                        @direction[:etape] = 1
                    end
                else
                    x = LARGEUR_ECRAN
                    @aliens.each {|col|
                        next if col.empty?
                        coord = col[0].coord
                        x = coord[0] if coord[0] < x
                    }
                    x -= 20
                    if x - PAS_ALIEN <= @SEMI_VERTICAL
                        @direction[:y] = PAS_ALIEN
                        @direction[:etape] = 1
                    end
                end
            else
                if @direction[:etape] == 1
                    if @direction[:vertical] < @SEMI_VERTICAL
                        @direction[:vertical] += PAS_ALIEN
                    else
                        @direction[:etape] = 2
                        @direction[:x] = @direction[:x] > 0 ? -PAS_ALIEN : PAS_ALIEN
                    end
                else
                    if @direction[:vertical] > 0
                        @direction[:vertical] -= PAS_ALIEN
                    else
                        @direction[:y] = 0
                        @direction[:vertical] = 0
                    end
                end
            end
        end

        def creerRangee x, y, nombre, couleur
            points = POINTS_ALIENS[couleur]
            (0...nombre).each {|col|
                alien = Alien.new @canevas, x, y, couleur, points
                @aliens[col] << alien
                x += 50
            }
        end

        def changerImages
            indice = @indImages[0] % 2
            @aliens.each {|col|
                col.each {|alien|
                    alien.image = indice
                }
            }
        end
    end
end
