module MODJeuElements
    require './constantes'
    require './jeu/elements/projectile'
    class ProjectileCanon < MODJeuElements::Projectile

        require './jeu/collisions'
        def initialize canevas
            super canevas, C_PROJECTILE_CANON, TAG_PROJECTILE_CANON
        end

        def deplacer
            c = coords
            col = MODJeu::Collisions.projectileCanon @canevas, *c
            unless col or c[3] < 0
                @canevas.jeu.move @id, 0, -PAS_PROJECTILE_CANON
                @canevas.jeu.after(
                    DELAI_DEPLACEMENT_PROJECTILE_CANON,
                    @deplacer,
                )
            else
                detruire
            end
        end
    end


    class Canon
        require './ressources'
        def initialize canevas
            @canevas = canevas
            @images = MODRessources::Images.new
            @image = @images.image :canon
            @projectile = ProjectileCanon.new canevas
            @statuts = {
                :projectile=>true,
                :mouvement=>true,
            }
        end

        def creer x:nil, y:nil
            x ||= LARGEUR_ECRAN / 2
            y ||= HAUTEUR_ZONE_JEU
            @id = @canevas.jeu.create(
                :image,
                x, y,
                :image=>@image,
                :tags=>TAG_CANON,
                :anchor=>'s',
            )
        end

        def coords
            @canevas.jeu.bbox @id
        end

        def deplacer direction
            return unless @statuts[:mouvement]
            c = coords
            if direction == DROITE and c[2] + PAS_CANON < LARGEUR_ECRAN
                @canevas.jeu.move @id, PAS_CANON, 0
            elsif direction == GAUCHE and c[0] - PAS_CANON > 0
                @canevas.jeu.move @id, -PAS_CANON, 0
            end
        end

        def tirer
            return if @projectile.actif or not @statuts[:projectile]
            x, y, x2, _ = coords
            ch = x + ((x2 - x) / 2)
            @projectile.creer ch, y
        end

        def infligerDegat
            imgDegat = @images.image :canon, indice:1
            @canevas.jeu.itemconfigure @id, :image=>imgDegat
            @canevas.jeu.after(
                TEMPS_DEGAT_CANON,
                lambda {@canevas.jeu.itemconfigure @id, {:image=>@image}}
            )
        end

        def detruire
            self.statuts = false
            imgExplosion = @images.image :canon, indice:2
            @canevas.jeu.itemconfigure(@id, :image=>imgExplosion)
            @canevas.jeu.after(
                TEMPS_EXPLOSION_CANON,
                lambda {@canevas.jeu.delete(@id)}
            )
        end

        def statuts=(bool)
            @statuts.each {|c, _| _statut(c, bool)}
        end

        def _statut nom, bool
            # raise TypeError unless [true, false].include? bool
            @statuts[nom] = bool
        end
        private :_statut

        def mouvement
            @statuts[:mouvement]
        end

        def mouvement=(bool)
            _statut :mouvement, bool
        end

        def projectile
            @statuts[:projectile]
        end

        def projectile=(bool)
            _statut :projectile, bool
        end
    end
end
