import random
import ressources
from constantes import *

_SENS = (
    # pas x, orig. x, dest. x, réf. fonc. comparaison
    (1, -20, LARGEUR_ECRAN + 20, float.__ge__), # Gauche vers droite
    (-1, LARGEUR_ECRAN + 20, -20, float.__le__), # Droite vers gauche
)

class Soucoupe :
    def __init__(self, canevas) :
        self._canevas = canevas
        self._images = ressources.Images()
        self._direction=dict(
            x=0,
            destx=0,
            comp=None,
            delai=0,
        )
        self._statuts = dict(
            mouvement=True,
            apparition=False,
        )
        self._image = self._images.image('soucoupe')
        self._id = self._canevas.jeu.create_image(
            -50, SOUCOUPE_Y,
            image=self._image,
            tags=TAG_SOUCOUPE,
        )
        self._actions = dict(
            apparaitre=None,
            deplacer=None,
        )
        self._actif = False
        self._points = POINTS_SOUCOUPE

    def _creer(self, x) :
        self._canevas.jeu.coords(self._id, x, SOUCOUPE_Y)

    def detruire(self) :
        self.mouvement = False
        img = self._images.image('soucoupe', 1)
        self._canevas.jeu.itemconfigure(self._id, image=img)
        self._canevas.jeu.after(
            TEMPS_EXPLOSION_SOUCOUPE,
            self._initPostDestruction,
        )

    def _initPostDestruction(self) :
        self._canevas.jeu.coords(
            self._id,
            self._direction['destx'],
            SOUCOUPE_Y
        )
        self._direction['x'] = self._direction['destx']
        self._canevas.jeu.itemconfigure(self._id, image=self._image)
        self.mouvement = True

    def _peutBouger(self, statut) :
        statut = bool(statut)
        if self._statuts['mouvement'] == statut :
            return
        self._statuts['mouvement'] = statut
        if statut :
            self._deplacer()
        else :
            self._canevas.jeu.after_cancel(self._actions['deplacer'])

    def _peutApparaitre(self, statut)  :
        statut = bool(statut)
        if self._statuts['apparition'] == statut :
            return
        self._statuts['apparition'] = statut
        if statut :
            if not self._actif :
                self._animer()
        else :
            self._canevas.jeu.after_cancel(self._actions['apparaitre'])

    def _animer(self) :
        if self._actif or not self._statuts['apparition'] :
            return
        sens = random.choice(_SENS)
        self._direction['x'] = sens[0]
        self._direction['destx'] = sens[2]
        self._direction['comp'] = sens[3]
        self._direction['delai'] = random.randint(*DELAIS_DEPLACEMENT_SOUCOUPE)
        self._creer(sens[1])
        delai = random.randint(*DELAIS_APPARITION_SOUCOUPE)
        self._actions['apparaitre'] = self._canevas.jeu.after(
            delai,
            self._transition
        )

    def _transition(self) :
        self._actif = True
        self._deplacer()

    def _deplacer(self) :
        if not self._statuts['mouvement'] :
            return
        coord = self._canevas.jeu.coords(self._id)
        comp = self._direction['comp']
        if not comp(coord[0], self._direction['destx']) :
            self._canevas.jeu.move(self._id, self._direction['x'], 0)
            self._actions['deplacer'] = self._canevas.jeu.after(
                self._direction['delai'],
                self._deplacer,
            )
        else :
            self._actif = False
            self._animer()

    def points(self) :
        return self._points

    apparition = property(fset=_peutApparaitre)
    mouvement = property(fset=_peutBouger)
