module MODJeuElements
    require './ressources'
    require './constantes'

    SENS = [
        # pas x, orig. x, dest. x, proc. comparaison
        [1, -20, LARGEUR_ECRAN + 20,  lambda {|a, b| a > b}], # Gauche vers droite
        [-1, LARGEUR_ECRAN + 20, -20, lambda {|a, b| a < b}], # Droite vers gauche
    ]

    class Soucoupe
        attr_reader :points
        def initialize canevas
            @canevas = canevas
            @images = MODRessources::Images.new
            @direction = {:x=>0, :destx=>0, :comp=>nil, :delai=>0}
            @statuts = {:mouvement=>true, :apparition=>false,}
            @actions = {:apparaitre=>nil, :deplacer=>nil,}
            @actif = false
            @points = POINTS_SOUCOUPE
            @image = @images.image :soucoupe
            @dds = Range.new *DELAIS_DEPLACEMENT_SOUCOUPE
            @das = Range.new *DELAIS_APPARITION_SOUCOUPE
            @procs = {
                :deplacer=>lambda {deplacer},
                :initPostDestruction=>lambda {initPostDestruction},
                :transition=>lambda {transition},
            }
            _creer
        end

        def detruire
            self.mouvement = false
            img = @images.image :soucoupe, indice:1
            @canevas.jeu.itemconfigure(@id, :image=>img)
            @canevas.jeu.after(
                TEMPS_EXPLOSION_SOUCOUPE,
                @procs[:initPostDestruction]
            )
        end

        def mouvement=(bool)
            return if @statuts[:mouvement] == bool
            @statuts[:mouvement] = bool
            if bool
                deplacer
            else
                @canevas.jeu.after_cancel @actions[:deplacer]
            end
        end

        def apparition=(bool)
            return if @statuts[:apparition] == bool
            @statuts[:apparition] = bool
            if bool
                animer unless @actif
            else
                @canevas.jeu.after_cancel @actions[:apparaitre]
            end
        end

        private
        def _creer
            @id = @canevas.jeu.create(
                :image,
                -50, SOUCOUPE_Y,
                :image=>@image,
                :tags=>TAG_SOUCOUPE,
            )
        end

        def creer x
            @canevas.jeu.coords @id, x, SOUCOUPE_Y
        end

        def initPostDestruction
            @canevas.jeu.coords(
                @id,
                @direction[:destx],
                SOUCOUPE_Y
            )
            @direction[:x] = @direction[:destx]
            @canevas.jeu.itemconfigure(@id, :image=>@image)
            self.mouvement = true
        end

        def animer
            return if @actif or not @statuts[:apparition]
            sens = SENS[rand(0..1)]
            @direction[:x] = sens[0]
            @direction[:destx] = sens[2]
            @direction[:comp] = sens[3]
            @direction[:delai] = rand @dds
            creer sens[1]
            delai = rand @das
            @actions[:apparaitre] = @canevas.jeu.after delai, @procs[:transition]
        end

        def transition
            @actif = true
            deplacer
        end

        def deplacer
            return unless @statuts[:mouvement]
            coord = @canevas.jeu.coords @id
            unless @direction[:comp].call(coord[0], @direction[:destx])
                @canevas.jeu.move @id, @direction[:x], 0
                @actions[:deplacer] = @canevas.jeu.after(
                    @direction[:delai],
                    @procs[:deplacer]
                )
            else
                @actif = false
                animer
            end
        end
    end
end
