import tkinter as tk
import ressources
from elements import texte
from constantes import *

_TAG_MESSAGE_CADRE = 'message_cadre'

class Messages :
    ''' Messages du jeu '''
    def __init__(self, canevas) :
        self._canevas = canevas
        self._coords = (100, 200, LARGEUR_ECRAN - 100, 320)
        images = ressources.Images()
        self._imageFond = images.image('fond_message')
        self._couleurs = (C_MESSAGE_TXT, C_MESSAGE_TXT_CONTOUR)
        self._actif = False
        self._initItems()

    def _initItems(self) :
        ''' Interne '''
        x, y, x2, y2 = self._coords
        polices = ressources.Polices()
        i = 0
        for xx in range(x, x2, 50) :
            i += 1
            self._canevas.create_image(
                xx, y,
                image=self._imageFond,
                tag=_TAG_MESSAGE_CADRE,
                anchor=tk.NW,
                state=tk.HIDDEN,
            )
        self._canevas.create_line(
            x+3, y+3, x2-5, y+3, x2-5, y2-5, x+3, y2-5, x+3, y+3,
            width=5,
            fill=C_MESSAGE_CADRE,
            tag=_TAG_MESSAGE_CADRE,
            state=tk.HIDDEN,
            stipple='gray75',
        )
        self._texte = texte.Texte(
            self._canevas,
            x+(x2-x)/2, y+(y2-y)/2,
            '',
            polices.police('message'),
            *self._couleurs
        )
        self._texte.cacher()

    @property
    def texte(self) :
        return self._texte.texte

    @texte.setter
    def texte(self, texte) :
        ''' Change le texte du message '''
        if '\n' in texte :
            parts = texte.splitlines()
            maxl = max(len(p) for p in parts)
            texte = '\n'.join(p.center(maxl) for p in parts)
        self._texte.texte = texte

    def afficher(self) :
        ''' Affiche le message '''
        self._actif = True
        self._canevas.tag_raise(_TAG_MESSAGE_CADRE, tk.ALL)
        self._canevas.itemconfigure(_TAG_MESSAGE_CADRE, state=tk.DISABLED)
        self._texte.focus()
        self._texte.montrer()

    def effacer(self) :
        ''' Efface le message '''
        self._actif = False
        self._texte.cacher()
        self._canevas.itemconfigure(_TAG_MESSAGE_CADRE, state=tk.HIDDEN)

    @property
    def actif(self) :
        return self._actif
