import tkinter as tk
import ressources
from elements import decors
from constantes import *

class Interface(tk.Frame) :
    ''' Base des interfaces '''
    def __init__(self, parent) :
        super().__init__(parent)
        self.grid()


class Canevas(tk.Canvas) :
    def __init__(self, parent, cnf={}, **dargs) :
        dargs.update(cnf)
        dargs.update((
            ('width', LARGEUR_ECRAN),
            ('bg', C_FOND),
            ('highlightthickness', 0),
        ))
        super().__init__(parent, dargs)
        self.grid()

    def _create(self, t, a, d) :
        '''
        Interne
        Surcharge de la méthode _create de tk.Canvas
        '''
        d.setdefault('state', tk.DISABLED)
        return super()._create(t, a, d)

    def lien(self, texte, y, bordure=True) :
        ''' Crée un lien texte '''
        polices = ressources.Polices()
        tid = self.create_text(
            LARGEUR_ECRAN / 2, y,
            text=texte,
            fill=C_LIEN_TXT,
            font=polices.police('lien'),
        )
        if bordure :
            box = self.bbox(tid)
            sep = 16
            fid = self.create_rectangle(
                box[0]-sep, box[1]-sep, box[2]+sep, box[3]+sep,
                width=3,
                outline=C_LIEN_TXT,
                dash=(3, 5, 2, 3),
            )
            self.tag_raise(tid, fid)


class CanevasDecors(Canevas, decors.Decors) :
    ''' Canevas incluant des méthodes de décorations '''
    pass


class CanevasJeuDecors(CanevasDecors) :
    ''' Canevas du jeu '''
    from time import time as timestamp
    def __init__(self, parent) :
        super().__init__(parent, height=HAUTEUR_ECRAN)
        self._pause = False
        self._varPause = tk.BooleanVar()
        self._commandes = {}
        self.focus_set()
        self.etoiles(y2=HAUTEUR_ECRAN, nombre=150)
        self.soleil()

    def _tms(self) :
        return int(self.timestamp() * 1000)

    def after(self, ms, func=None, *args) :
        '''
        Surcharge de la méthode after de tkinter afin de gérer la suspension
        du jeu
        '''
        if self._pause :
            self.wait_variable(self._varPause)
        aid = super().after(ms, func, *args)
        rid = self._tclCommands[-1]
        self._commandes[rid] = (ms, self._tms(), func, args)
        return aid

    def deletecommand(self, name, da=True) :
        '''
        Surcharge de la méthode deletecommand de tkinter afin de gérer la
        suspension du jeu
        '''
        if da :
            try :
                del(self._commandes[name])
            except KeyError :
                pass
        super().deletecommand(name)

    def suspendreActions(self) :
        ''' Suspend les actions prévues par la méthode after '''
        self._pause = True
        for n in tuple(self._commandes) :
            self.deletecommand(n, False)

    def reprendreActions(self) :
        ''' Reprend les actions prévues par la méthode after '''
        self._pause = False
        commandes = tuple(self._commandes.values())
        self._commandes.clear()
        for ms, ts, f, args in commandes :
            ms -= self._tms() - ts
            self.after(ms, f, *args)

    def destroy(self) :
        ''' Surcharge de la méthode destroy de tkinter '''
        self._pause = False
        super().destroy()


class CanevasScore(CanevasDecors) :
    ''' Canevas du score '''
    def __init__(self, parent) :
        super().__init__(
            parent,
            height=HAUTEUR_ECRAN,
        )
        self.focus_set()
        self.etoiles(y2=HAUTEUR_ECRAN, nombre=200)


class CanevasCommandes(CanevasDecors) :
    ''' Canevas des commandes '''
    def __init__(self, parent) :
        super().__init__(
            parent,
            height=HAUTEUR_ECRAN,
        )
        self.focus_set()
        self.etoiles(y2=HAUTEUR_ECRAN, nombre=100)


class CanevasInfos(Canevas) :
    ''' Canevas des informations du jeu '''
    def __init__(self, parent) :
        super().__init__(
            parent,
            height=HAUTEUR_ZONE_INFO,
            takefocus=False,
        )
        # Bordure de séparation avec le canevas du jeu
        self.create_rectangle(
            0, HAUTEUR_ZONE_INFO - 2, LARGEUR_ECRAN, HAUTEUR_ZONE_INFO,
            fill=C_SEP_CAN
        )


class CanevasMenu(CanevasDecors) :
    ''' Canevas du menu '''
    def __init__(self, parent) :
        super().__init__(parent, height=HAUTEUR_ECRAN)
        self.focus_set()
        self.etoiles(y2=HAUTEUR_ECRAN)
        self.soleil()


class CanevasJeu :
    ''' Canevas jeu et infos du jeu '''
    def __init__(self, parent) :
        self._parent = parent
        self._canevas = dict(
            infos=CanevasInfos(self._parent),
            jeu=CanevasJeuDecors(self._parent)
        )
        bid = self.jeu.create_rectangle(
            0, HAUTEUR_ZONE_JEU-5, LARGEUR_ECRAN, HAUTEUR_ZONE_JEU,
            tags=TAG_BASE,
            fill=C_BASE_CANON,
        )
        self.jeu.tag_lower(bid, tk.ALL)
        self.jeu.focus()

    infos = property(lambda s: s._canevas['infos'])
    jeu = property(lambda s: s._canevas['jeu'])
