module MODBasesCanevas
    require './elements/decors'

    class Canevas < TkCanvas
        def initialize parent, dargs
            dargs.update({
                :width=>LARGEUR_ECRAN,
                :bg=>C_FOND,
                :highlightthickness=>0,
            })
            super parent, dargs
            grid
        end

        def create type, *coords, **attrs
            attrs[:state] = :disabled unless attrs.key?(:state)
            super type, coords << attrs
        end

        def lien texte, y, bordure:false
            require './ressources'
            # Crée un lien texte
            polices = MODRessources::Polices.new
            tid = create(
                :text,
                LARGEUR_ECRAN / 2, y,
                :text=>texte,
                :fill=>C_LIEN_TXT,
                :font=>polices.police(:lien),
            )
            if bordure
                box = bbox tid
                sep = 16
                rid = create(
                    :rectangle,
                    box[0]-sep, box[1]-sep, box[2]+sep, box[3]+sep,
                    :width=>3,
                    :outline=>C_LIEN_TXT,
                    :dash=>[3, 5, 2, 3],
                )
                raise tid, rid
            end
        end
    end


    class CanevasDecors < Canevas
        # Canevas incluant des méthodes de décorations
        include MODDecors
    end


    class CanevasJeuDecors < CanevasDecors
        require 'time'
        def initialize parent
            super(parent, :height=>HAUTEUR_ECRAN)
            @commandes = {}
            @encours = []
            @pause = false
            @varPause = TkVariable.new
            focus
            etoiles HAUTEUR_ECRAN, nombre:150
            soleil
        end

        def tms
            (Time.now.to_f * 1000).to_i
        end
        private :tms

        def after ms, proc
            # Surcharge
            @varPause.wait if @pause
            aid = super ms, proc
            @commandes[@cmdId] = [aid, tms, ms, proc]
            aid
        end

        def install_cmd cmd
            # Surcharge
            @cmdId = super cmd
        end

        def uninstall_cmd id
            # Surcharge
            @commandes.delete id
            super id
        end

        def suspendreActions
            # Suspend les actions prévues par la méthode after
            @commandes.each{|id, v|
                after_cancel(v[0])
                @encours << v[1..3]
            }
            @pause = true
        end

        def reprendreActions
            # Reprend les actions prévues par la méthode after
            @pause = false
            @encours.each{|v|
                ms = v[1] - (tms - v[0])
                after ms, v[2]
            }
            @encours.clear
        end

        def destroy
            # Surcharge
            @pause = false
            super
        end
    end


    class CanevasInfos < CanevasDecors
        # Canevas des informations du jeu
        def initialize parent
            super(
                parent,
                :height=>HAUTEUR_ZONE_INFO,
                :takefocus=>false,
            )
            # Bordure de séparation avec le canevas du jeu
            create(
                :rectangle,
                0, HAUTEUR_ZONE_INFO - 2, LARGEUR_ECRAN, HAUTEUR_ZONE_INFO,
                :fill=>C_SEP_CAN
            )
        end
    end
end


module MODWidgets
    require 'tk'
    require './constantes'
    class Interface < TkFrame
        # Base des interfaces du jeu
        def initialize parent
            super parent
            grid
        end
    end


    class CanevasMenu < MODBasesCanevas::CanevasDecors
        # Canevas du menu
        def initialize parent
            super parent, :height=>HAUTEUR_ECRAN
            focus
            etoiles HAUTEUR_ECRAN
            soleil
        end
    end


    class CanevasJeu
        # Canevas jeu et infos du jeu
        def initialize parent
            @parent = parent
            @canevas = {
                :infos=>MODBasesCanevas::CanevasInfos.new(parent),
                :jeu=>MODBasesCanevas::CanevasJeuDecors.new(parent)
            }
            bid = jeu.create(
                :rectangle,
                0, HAUTEUR_ZONE_JEU-5, LARGEUR_ECRAN, HAUTEUR_ZONE_JEU,
                :tags=>TAG_BASE,
                :fill=>C_SEP_CAN,
            )
            jeu.lower(bid, :all)
            jeu.focus
        end

        def jeu
            @canevas[:jeu]
        end

        def infos
            @canevas[:infos]
        end
    end


    class CanevasScore < MODBasesCanevas::CanevasDecors
        def initialize parent
            super parent, :height=>HAUTEUR_ECRAN
            focus
            etoiles HAUTEUR_ECRAN
        end
    end


    class CanevasCommandes < MODBasesCanevas::CanevasDecors
        def initialize parent
            super parent, :height=>HAUTEUR_ECRAN
            focus
            etoiles HAUTEUR_ECRAN, :nombre=>100
        end
    end
end
